# Defines functions for this feature
# XXX password must be set when enabling!

do '../web-lib.pl';
&init_config();
do '../ui-lib.pl';
$mysql::use_global_login = 1;
&foreign_require("mysql", "mysql-lib.pl");

# feature_name()
# Returns a short name for this feature
sub feature_name
{
return $text{'feat_name'};
}

# mailbox_inputs(&user, new, &domain)
# Returns HTML for additional inputs on the mailbox form. These should be
# formatted to appear inside a table.
sub mailbox_inputs
{
local ($user, $new, $dom) = @_;
return undef if ($dom && !$dom->{'mysql'});
return undef if ($dom && $user && $dom->{'user'} eq $user->{'user'});
local $muser;
local @mdbs;
if (!$new) {
	local @users = &list_users();
	local $un = &user_name($user, $dom);
	($muser) = grep { $_->[1] eq $un } @users;
	if ($muser) {
		@mdbs = &list_user_dbs($un);
		}
	}
local $ui_table_cols = 2;
return &ui_table_row($text{'mail_mysql'},
		     &ui_radio($module_name,
			       $muser ? 1 : 0,
			       [ [ 1, $text{'yes'} ],
				 [ 0, $text{'no'} ] ]))."\n".
       &ui_table_row($text{'mail_mysqldbs'},
		     &ui_select($module_name."_dbs",
				\@mdbs,
				[ map { [ $_, $_ ] } &list_databases($dom) ],
				5, 1));
}

# mailbox_validate(&user, &olduser, &in, new, &domain)
# Validates inputs generated by mailbox_inputs, and returns either undef on
# success or an error message
sub mailbox_validate
{
local ($user, $olduser, $in, $new, $dom) = @_;
return undef if ($dom && !$dom->{'mysql'});
return undef if ($dom && $dom->{'user'} eq $olduser->{'user'});
if ($in->{$module_name}) {
	local @users = &list_users();
	local $un = &user_name($user, $dom);
	local ($muser) = grep { $_->[1] eq $un } @users;

	# Make sure MySQL user doesn't clash
	if ($new || $user->{'user'} ne $olduser->{'user'}) {
		return &text('mail_clash', $user->{'user'}) if ($muser);
		}
	# Make sure a password is given if needed
	if ($user->{'passmode'} != 3 && !$muser) {
		return $text{'mail_pass'};
		}
	}
return undef;
}

# mailbox_save(&user, &olduser, &in, new, &domain)
# Updates the user based on inputs generated by mailbox_inputs
sub mailbox_save
{
local ($user, $olduser, $in, $new, $dom) = @_;
return undef if ($dom && !$dom->{'mysql'});
return undef if ($dom && $dom->{'user'} eq $olduser->{'user'});
local @users = &list_users();
local $muser;
if (!$new) {
	($muser) = grep { $_->[1] eq &user_name($olduser, $dom) } @users;
	}

local @hosts = split(/\s+/, $virtual_server::config{'mysql_hosts'});
@hosts = ( 'localhost' ) if (!@hosts);
local @mdbs = split(/\0/, $in->{$module_name."_dbs"});
local ($h, $db);

if ($in->{$module_name} && !$muser) {
	# Need to create
	local $un = &user_name($user, $dom);
	foreach $h (@hosts) {
		&mysql::execute_sql_logged($mysql::master_db,
			"insert into user (host, user, password) values ('$h', '$un', password('$user->{'plainpass'}'))");
		}
	foreach $h (@hosts) {
		foreach $db (@mdbs) {
			&mysql::execute_sql_logged($mysql::master_db, "insert into db (host, db, user, select_priv, insert_priv, update_priv, delete_priv, create_priv, drop_priv, grant_priv, references_priv, index_priv, alter_priv) values ('$h', '$db', '$un', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y')");
			}
		}
	}
elsif (!$in->{$module_name} && $muser) {
	# Need to delete
	local $un = &user_name($olduser, $dom);
	&mysql::execute_sql_logged($mysql::master_db,
		"delete from user where user = '$un'");
	&mysql::execute_sql_logged($mysql::master_db,
		"delete from db where user = '$un'");
	}
elsif ($in->{$module_name} && $muser) {
	# Need to update
	local $un = &user_name($user, $dom);
	local $oun = &user_name($olduser, $dom);
	if ($olduser->{'user'} ne $user->{'user'}) {
		# Rename user
		&mysql::execute_sql_logged($mysql::master_db,
			"update user set user = '$un' where user = '$oun'");
		&mysql::execute_sql_logged($mysql::master_db,
			"update db set user = '$un' where user = '$oun'");
		}
	if ($user->{'passmode'} == 3) {
		# Change user's password
		&mysql::execute_sql_logged($mysql::master_db,
			"update user set password = password('$user->{'plainpass'}') where user = '$un'");
		}
	# Re-set all databases
	&mysql::execute_sql_logged($mysql::master_db,
		"delete from db where user = '$un'");
	foreach $h (@hosts) {
		foreach $db (@mdbs) {
			&mysql::execute_sql_logged($mysql::master_db, "insert into db (host, db, user, select_priv, insert_priv, update_priv, delete_priv, create_priv, drop_priv, grant_priv, references_priv, index_priv, alter_priv) values ('$h', '$db', '$un', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y')");
			}
		}
	}
&mysql::execute_sql_logged($mysql::master_db, 'flush privileges');
}

# mailbox_delete(&user, &domain)
# Removes any extra features for this user
sub mailbox_delete
{
local ($user, $dom) = @_;
return undef if ($dom && !$dom->{'mysql'});
return undef if ($dom && $dom->{'user'} eq $olduser->{'user'});
local @users = &list_users();
local $un = &user_name($user, $dom);
local ($muser) = grep { $_->[1] eq $un } @users;
if ($muser) {
	# Remove MySQL access
	&mysql::execute_sql_logged($mysql::master_db,
		"delete from user where user = '$un'");
	&mysql::execute_sql_logged($mysql::master_db,
		"delete from db where user = '$un'");
	&mysql::execute_sql_logged($mysql::master_db, 'flush privileges');
	}
}

# mailbox_header(&domain)
# Returns a column header for the user display, or undef for none
sub mailbox_header
{
if ($_[0]->{'mysql'}) {
	@column_users = &list_users();
	return $text{'mail_header'};
	}
else {
	return undef;
	}
}

# mailbox_column(&user, &domain)
# Returns the text to display in the column for some user
sub mailbox_column
{
local ($user, $dom) = @_;
local $un = &user_name($user, $dom);
local ($muser) = grep { $_->[1] eq $un } @column_users;
return $muser ? $text{'yes'} : $text{'no'};
}

# user_name(&user, &dom)
sub user_name
{
return length($_[0]->{'user'}) > 16 ? substr($_[0]->{'user'}, 0, 16)
				    : $_[0]->{'user'};
}

sub list_users
{
local $d = &mysql::execute_sql($mysql::master_db, "select * from user order by user");
return @{$d->{'data'}};
}

# list_user_dbs(username)
sub list_user_dbs
{
local $d = &mysql::execute_sql($mysql::master_db, "select db from db where user = '$_[0]'");
return map { $_->[0] } @{$d->{'data'}};
}

# list_databases(&domain)
sub list_databases
{
local $tmpl = &virtual_server::get_template($_[0]->{'template'});
local $wild = &virtual_server::substitute_template(
		$tmpl->{'mysql_wild'}, $_[0]);
local @alldbs = &mysql::list_databases();
local @dbs;
if ($wild && $wild ne $_[0]->{'db'}) {
	$wild =~ s/\%/\.\*/g;
	$wild =~ s/_/\./g;
	push(@dbs, grep { /^$wild$/i } @alldbs);
	}
push(@dbs, split(/\s+/, $_[0]->{'db_mysql'}));
return &unique(@dbs);
}

1;

